<?php declare(strict_types=1);

use GuzzleHttp\Client as Client;
use GuzzleHttp\Exception;
use GuzzleHttp\Exception\ServerException;
use GuzzleHttp\Exception\ClientException;

namespace nogueiracodes\RaspberryBot\Bot\Helpers;

use Analog\Analog;
use Exception;
use InvalidArgumentException;
use Zttp\Zttp;

class CatAPI {

    private $headers;


    private $baseURL = "https://api.thecatapi.com/v1/";


    private $hardLimit = 25;

    public function __construct()
    {
        Analog::info('Helpers: Loaded new instance of CatAPI!');

        if (isset($_ENV['CAT_API_KEY']))
        {
            $this->headers = [
                'x-api-key' => $_ENV['CAT_API_KEY']
            ];
        }
        else
        {  
            $this->hardLimit = 1;
            Analog::urgent('CatAPI ERROR: Failed to load Cat API key. Results limited to 1 picture. It is recommended set an API key.');
        }
    }

    public function getCatPicture($count = 1, $size = "small"): string {
        
        Analog::debug('CatAPI: Fetching cat picture with count ' . $count . ' and size ' . $size);

        // anti smart-ass device
        if ($count > 0 && $count <= $this->hardLimit)
        {
            Analog::error('CatAPI: Invalid arguments: Count of ' . $count . ' is over the hard limit of ' . $this->hardLimit);
            throw new Exception("Unacceptable quantity: " . $count . ". Please try again with no more than " . $this->hardLimit . " picture(s).");
        }

        if (!empty($this->headers))
        {
            $catPic = Zttp::withHeaders($this->headers)->get($this->baseURL . 'images/search', [
                'limit' => $count,
                'size' => $size,
                'has_breeds' => false
            ]);
        }
        else
        {
            $catPic = Zttp::get($this->baseURL . 'images/search', [
                'limit' => $count,
                'size' => $size,
                'has_breeds' => false
            ]);
        }

        if ($catPic->isSuccess())
        {

            $results = $catPic->json();
            $pics = [];

            if (count($results) == 1)
            {
                Analog::debug('CatAPI: Fetched single cat picture.');
                return $results[0]['url'];
            }

            foreach($results as $result)
            {  
                Analog::debug('CatAPI: Fetched multiple cat pictures; Splitting into EOL divided string.');
                array_push($pics, $result['url']);
            }

            return implode(PHP_EOL, $pics);
            
        }
        else
        {
            Analog::error('CatAPI: Network error. Failed to fetch cat picture. Request status code: ' . $catPic->status());
            throw new Exception('Sorry, but I couldn\'t get you a good cat picture this time. Try again later!');
        }

    }

}